// src/internal/persistent-port.ts
import browser from "webextension-polyfill";
var createPersistentPort = (name = "") => {
  let port;
  const queue = [];
  const onMessageListeners = /* @__PURE__ */ new Set();
  const handleMessage = (message, port2) => {
    console.log("raw message", message);
    if (message.status === "undeliverable") {
      if (!queue.some((m) => m.message.messageID === message.data.message.messageID))
        queue.push(message.data);
      return;
    }
    if (message.status === "deliverable") {
      [...queue].forEach((queuedMsg) => {
        if (queuedMsg.resolvedDestination === message.deliverableTo) {
          port2.postMessage(queuedMsg.message);
          queue.splice(queue.indexOf(queuedMsg), 1);
        }
      });
      return;
    }
    onMessageListeners.forEach((cb) => cb(message, port2));
  };
  const connect = () => {
    console.log("port conn", new Date().toISOString());
    port = browser.runtime.connect({ name });
    port.onMessage.addListener(handleMessage);
    port.onDisconnect.addListener(connect);
    queue.forEach(({ message }) => port.postMessage(message));
  };
  connect();
  return {
    onMessage(cb) {
      onMessageListeners.add(cb);
    },
    postMessage(message) {
      port.postMessage(message);
    }
  };
};

export {
  createPersistentPort
};
